package org.rcisoft.core.runner;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.core.auth.config.CyServiceAuthConfig;
import org.rcisoft.core.auth.feign.CyAuthServerFeign;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyRsaKeyHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.http.HttpStatus;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.IOException;

/**
 * Created by lcy on 18/4/9.
 *
 * 获取用户 client/user  公钥
 */
@Component
@Slf4j
public class CyAuthClientRunner implements CommandLineRunner {

    @Autowired
    private CyServiceAuthConfig rcServiceAuthConfig;

    @Autowired
    private CyAuthServerFeign rcAuthServerFeign;

    @Override
    public void run(String... args) throws Exception {
        log.info("初始化加载服务端pubKey");
        try {
            refreshServicePubKey();
        }catch(Exception e){
            log.error("初始化加载服务端pubKey失败,1分钟后自动重试!",e);
        }
    }


    /**
     * 刷新service 公钥
     */
    @Scheduled(cron = "0 0/10 * * * ?")
    public void refreshServicePubKey() throws IOException{
        CyResult resp = rcAuthServerFeign.getServicePublicKey(rcServiceAuthConfig.getClientId(), rcServiceAuthConfig.getClientSecret());
        if (resp.getCode() == HttpStatus.OK.value()) {
            Object data = resp.getData();
            byte[] re = CyRsaKeyHelper.toBytes(data.toString());
            this.rcServiceAuthConfig.setPubKeyByte(re);
        }
    }

}
