package org.rcisoft.core.util;


import lombok.extern.slf4j.Slf4j;
import org.rcisoft.core.auth.config.CyServiceAuthConfig;
import org.rcisoft.core.auth.feign.CyAuthServerFeign;
import org.rcisoft.core.result.CyResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * Created by lcy on 18/4/9.
 */
@Component
@Slf4j
@EnableScheduling
public class CyServiceAuthUtil {

    @Autowired
    private CyServiceAuthConfig rcServiceAuthConfig;

    @Autowired
    private CyAuthServerFeign rcAuthServerFeign;
    /**
     * 允许调用的 client
     */
    private List<String> allowedClient;

    private String clientToken;


    @Scheduled(cron = "0 0/10 * * * ?")
    public void refreshClientToken() {
        log.debug("refresh client token.....");
        CyResult resp = rcAuthServerFeign.getAccessToken(rcServiceAuthConfig.getClientId(), rcServiceAuthConfig.getClientSecret());
        if (resp.getCode() == 200)
            this.clientToken = resp.getData().toString();
    }


    public String getClientToken() {
        if (this.clientToken == null) {
            this.refreshClientToken();
        }
        return clientToken;
    }

}
