package org.rcisoft.core.util;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.poi.POIXMLDocument;
import org.apache.poi.hslf.usermodel.HSLFSlideShow;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hwpf.extractor.WordExtractor;
import org.apache.poi.openxml4j.opc.OPCPackage;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xslf.usermodel.XMLSlideShow;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.springframework.web.multipart.MultipartFile;
import ucar.nc2.util.IO;


import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.*;

@Slf4j
public class FileReadableUtil {

    public static void main(String[] args)  {
         String filePath = "F:\\桌面整理\\文件归纳\\题库模板\\第二阶段测试用例.xlsx";
        fileReadable(filePath,(long)100,100);
    }

    private static void closeOpcPackage(OPCPackage opcPackage){
        try{
            if(opcPackage != null){
                opcPackage.flush();
                opcPackage.close();
            }
        }catch(Exception e){
            log.error("关闭 OPCPackage 输出流错误！", e);
        }
    }
    private static void closeOtherStream(Closeable stream){
        try{
            if(stream != null){
                stream.close();
            }
        }catch(Exception e){
            log.error("关闭流错误！", e);
        }
    }

    private static Integer docReadable(String path){
        Integer pages;
        InputStream is = null;
        try {
            is = new FileInputStream(new File(path));
            WordExtractor ex = new WordExtractor(is);
            pages = ex.getSummaryInformation().getPageCount();//总页数
        } catch (Exception e) {
            log.error("文件不可读---"+e);
            throw new ServiceException(ResultServiceEnums.FILE_UNREADABLE);
        }finally {
            FileReadableUtil.closeOtherStream(is);
        }
        return pages;
    }

    private static Integer excelReadable(String path){
        Workbook wb = null;
        try {
            File excel = new File(path);
            FileInputStream fis = new FileInputStream(excel);
            if("xls".equalsIgnoreCase(path.substring(path.lastIndexOf(".")+1)))
                wb = new HSSFWorkbook(fis);
            if("xlsx".equalsIgnoreCase(path.substring(path.lastIndexOf(".")+1)))
                wb = new XSSFWorkbook(fis);
            wb.getSheetAt(0).getLastRowNum();
        } catch (Exception e) {
            log.error("文件不可读---"+e);
            throw new ServiceException(ResultServiceEnums.FILE_UNREADABLE);
        }finally {
            FileReadableUtil.closeOtherStream(wb);
        }
        return 1;
    }


    private static Integer pptReadable(String path){
        Integer pages;
        InputStream is = null;
        HSLFSlideShow slideShow = null;
        try {
            is =new FileInputStream(new File(path));
            slideShow = new HSLFSlideShow(is);
            pages = slideShow.getSlides().size();
        } catch (Exception e) {
            log.error("文件不可读---"+e);
            throw new ServiceException(ResultServiceEnums.FILE_UNREADABLE);
        }finally {
            FileReadableUtil.closeOtherStream(slideShow);
            FileReadableUtil.closeOtherStream(is);
        }
        return pages;
    }


    private static Integer pdfReadable(String path){
        Integer pages;
        PDDocument doc = null;
        try {
            doc = PDDocument.load(new File(path));
            pages = doc.getDocumentCatalog().getPages().getCount();
        } catch (Exception e) {
            log.error("文件不可读---"+e);
            throw new ServiceException(ResultServiceEnums.FILE_UNREADABLE);
        }finally {
            try{
                if(doc != null){
                    doc.close();
                }
            }catch(Exception e){
                log.error("关闭 PDDocument 输出流错误！", e);
            }
            FileReadableUtil.closeOtherStream(doc);
        }
        return pages;
    }
    private static Integer docxReadable(String path){
        Integer pages ;
        OPCPackage opcPackage = null;
        try {
            opcPackage = POIXMLDocument.openPackage(path);
            XWPFDocument docx = new XWPFDocument(opcPackage);
            pages = docx.getProperties().getExtendedProperties().getUnderlyingProperties().getPages();
        } catch (Exception e) {
            log.error("文件不可读---"+e);
            throw new ServiceException(ResultServiceEnums.FILE_UNREADABLE);
        }finally {
            FileReadableUtil.closeOpcPackage(opcPackage);
        }
        return pages;
    }
    private static Integer pptxReadable(String path){
        Integer pages;
        XMLSlideShow pptxfile = null;
        OPCPackage opcPackage = null;
        try {
            opcPackage = POIXMLDocument.openPackage(path);
            pptxfile = new XMLSlideShow(opcPackage);
            pages = pptxfile.getSlides().size();
        } catch (Exception e) {
            log.error("文件不可读---"+e);
            throw new ServiceException(ResultServiceEnums.FILE_UNREADABLE);
        } finally {
            FileReadableUtil.closeOpcPackage(opcPackage);
        }
        return pages;
    }


    public static void fileReadable(String path,Long fileMaxSize,Integer fileMaxPages) {
        LogUtil.fileChangeLog("-----------FileReadableUtil-路径------------"+path);
        File file = new File(path);
        Long size = file.length();
        if (size > (long)(1024 * 1024 * fileMaxSize)){
            throw new ServiceException(400,"该文件超过最大"+fileMaxSize+"M限制");
        }

        Integer pages = -1;

        switch (path.substring(path.lastIndexOf(".") + 1)) {
            case "doc":
                pages = FileReadableUtil.docReadable(path);
                break;
            case "ppt":
                pages = FileReadableUtil.pptReadable(path);
                break;
            case "pdf":
                pages = FileReadableUtil.pdfReadable(path);
                break;
            case "docx":
                pages = FileReadableUtil.docxReadable(path);
                break;
            case "pptx":
                pages = FileReadableUtil.pptxReadable(path);
                break;
            case "xls":
                pages = FileReadableUtil.excelReadable(path);
                break;
            case "xlsx":
                pages = FileReadableUtil.excelReadable(path);
                break;
            default:
                break;
        }

        if (pages != -1 && pages > fileMaxPages)  {
            throw new ServiceException(400,"该文件超过最多"+fileMaxPages+"页限制");
        }

//        if (!CheckFileHeaderUtil.toCheck(path)){
//            throw new ServiceException(ResultServiceEnums.FILE_UNREADABLE);
//        }
    }

    public  File TXTHandler(File file) {
        //或GBK
        String code = "gb2312";
        byte[] head = new byte[3];
        try {
            InputStream inputStream = new FileInputStream(file);
            inputStream.read(head);
            if (head[0] == -1 && head[1] == -2) {
                code = "UTF-16";
            } else if (head[0] == -2 && head[1] == -1) {
                code = "Unicode";
            } else if (head[0] == -17 && head[1] == -69 && head[2] == -65) {
                code = "UTF-8";
            }
            inputStream.close();
            System.out.println(code);
            if (code.equals("UTF-8")) {
                return file;
            }
            String str = FileUtils.readFileToString(file, code);
            FileUtils.writeStringToFile(file, str, "UTF-8");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return file;
    }

    public static int imgReadable(MultipartFile f){
        try{
//            File f = new File(path);
//            FileInputStream fi = new FileInputStream(f);
            FileInputStream fi = (FileInputStream)f.getInputStream();
            try{
                BufferedImage sourceImg =ImageIO.read(fi);//判断图片是否损坏
                int picWidth= sourceImg.getWidth(); //确保图片是正确的（正确的图片可以取得宽度）
            }catch (Exception e) {
                fi.close();//关闭IO流才能操作图片
                throw new ServiceException(ResultServiceEnums.FILE_UNREADABLE);
            }finally{
                fi.close();//最后一定要关闭IO流
            }
        }catch (IOException e  ) {
            throw new ServiceException("读取io流失败");
        }
        return 1;
    }
    public static void imgReadable(String path){
        try{
            File f = new File(path);
            FileInputStream fi = new FileInputStream(f);
            try{
                BufferedImage sourceImg =ImageIO.read(fi);//判断图片是否损坏
                int picWidth= sourceImg.getWidth(); //确保图片是正确的（正确的图片可以取得宽度）
            }catch (Exception e) {
                fi.close();//关闭IO流才能操作图片
                throw new ServiceException(ResultServiceEnums.FILE_UNREADABLE);
            }finally{
                fi.close();//最后一定要关闭IO流
            }
        }catch (IOException e  ) {
            throw new ServiceException("读取io流失败");
        }
    }

}
